#include "glcdtimers_objects.h"

// Glcd module connections
char GLCD_DataPort at PORTD;
sbit GLCD_CS1 at LATJ0_bit;
sbit GLCD_CS2 at LATJ1_bit;
sbit GLCD_RS at LATJ2_bit;
sbit GLCD_RW at LATJ3_bit;
sbit GLCD_EN at LATJ4_bit;
sbit GLCD_RST at LATJ5_bit;
sbit GLCD_CS1_Direction at TRISJ0_bit;
sbit GLCD_CS2_Direction at TRISJ1_bit;
sbit GLCD_RS_Direction at TRISJ2_bit;
sbit GLCD_RW_Direction at TRISJ3_bit;
sbit GLCD_EN_Direction at TRISJ4_bit;
sbit GLCD_RST_Direction at TRISJ5_bit;
// End Glcd module connections

// Touch Panel module connections
sbit DriveA at LATJ6_bit;
sbit DriveB at LATJ7_bit;
sbit DriveA_Direction at TRISJ6_bit;
sbit DriveB_Direction at TRISJ7_bit;
// End Touch Panel module connections

// Global variables
unsigned int Xcoord, Ycoord;
char PenDown;
void *PressedObject;
static void InitializeTouchPanel() {
  CMCON  |= 0x07;
  //ADCON1 |= 0x0D;
  ADCON1 |= 0x0C; // AN4
  ADC_Init();
  Glcd_Init();                                             // Initialize GLCD
  Glcd_Fill(0);                                            // Clear GLCD

  TP_Init(128, 64, 0, 1);                                  // Initialize touch panel
  TP_Set_ADC_Threshold(900);                              // Set touch panel ADC threshold

  PenDown = 0;
  PressedObject = 0;
}

void Calibrate() {

  Glcd_Dot(0,63,1);                                        // Draw bottom left dot
  Glcd_Write_Text("TOUCH BOTTOM LEFT",12,3,1);
  TP_Calibrate_Bottom_Left();                              // Calibration of bottom left corner
  Delay_ms(1000);

  Glcd_Dot(0,63,0);                                        // Clear bottom left dot
  Glcd_Dot(127,0,1);                                       // Draw upper right dot
  Glcd_Write_Text("                 ",12,3,1);
  Glcd_Write_Text("TOUCH UPPER RIGHT",12,4,1);
  TP_Calibrate_Upper_Right();                              // Calibration of upper right corner

  Delay_ms(1000);
}


// Bitmaps

/////////////////////////
  TScreen *CurrentScreen;

  TScreen                Screen1;
  TLabel                 Label1;
  char Label1_Caption[8] = "Timer1";

  TLabel                 Label2;
  char Label2_Caption[8] = "Timer2";

  TLabel                 Label3;
  char Label3_Caption[8] = "Idle";

  TLabel                 Label4;
  char Label4_Caption[8] = "ADC5";

  TLabel                 Label5;
  char Label5_Caption[8] = "ADC3";

  TLabel                 Label6;
  char Label6_Caption[8] = "  RTC";

  TLabel                 Label7;
  char Label7_Caption[8] = "UART";

  TLabel                 Label13;
  char Label13_Caption[3] = " 0";
  
  TLabel                 Label8;
  char Label8_Caption[3] = " 1";

  TLabel                 Label9;
  char Label9_Caption[3] = " 2";

  TLabel                 Label10;
  char Label10_Caption[3] = " 3";

  TLabel                 Label11;
  char Label11_Caption[3] = " 4";

  TLabel                 Label12;
  char Label12_Caption[3] = " 5";

  TCaptionButton         CaptionButton1;
  char CaptionButton1_Caption[7] = "START";

  TLabel                 *Screen0_Labels[13];
  TCaptionButton         *Screen0_CaptionButtons[1];



static void InitializeObjects() {
  Screen1.LabelsCount               = 13;
  Screen1.Labels                    = Screen0_Labels;
  Screen1.CaptionButtonsCount       = 1;
  Screen1.CaptionButtons            = Screen0_CaptionButtons;
  Screen0_Labels[0]        = &Label1;
  Screen0_Labels[1]        = &Label2;
  Screen0_Labels[2]        = &Label3;
  Screen0_Labels[3]        = &Label4;
  Screen0_Labels[4]        = &Label5;
  Screen0_Labels[5]        = &Label6;
  Screen0_Labels[6]        = &Label7;
  Screen0_Labels[7]        = &Label8;
  Screen0_Labels[8]        = &Label9;
  Screen0_Labels[9]        = &Label10;
  Screen0_Labels[10]        = &Label11;
  Screen0_Labels[11]        = &Label12;
  Screen0_Labels[12]        = &Label13;
  
  Screen0_CaptionButtons[0]       = &CaptionButton1;

  Label1.OwnerScreen = &Screen1;
  Label1.Left        = 0;
  Label1.Top         = 0;
  Label1.Width       = 48;
  Label1.Height      = 8;
  Label1.Caption     = Label1_Caption;
  Label1.FontName    = Font_Glcd_Character8x7;
  Label1.FontWidth   = 8;
  Label1.FontHeight  = 7;
  Label1.FontOffset  = 32;
  Label1.FontColor   = _clDraw;
  Label1.Visible     = 1;
  Label1.OnUpPtr     = 0;
  Label1.OnDownPtr   = 0;
  Label1.OnClickPtr  = 0;
  Label1.OnPressPtr  = 0;

  Label2.OwnerScreen = &Screen1;
  Label2.Left        = 0;
  Label2.Top         = 8;
  Label2.Width       = 48;
  Label2.Height      = 8;
  Label2.Caption     = Label2_Caption;
  Label2.FontName    = Font_Glcd_Character8x7;
  Label2.FontWidth   = 8;
  Label2.FontHeight  = 7;
  Label2.FontOffset  = 32;
  Label2.FontColor   = _clDraw;
  Label2.Visible     = 1;
  Label2.OnUpPtr     = 0;
  Label2.OnDownPtr   = 0;
  Label2.OnClickPtr  = 0;
  Label2.OnPressPtr  = 0;

  Label4.OwnerScreen = &Screen1;
  Label4.Left        = 0;
  Label4.Top         = 16;
  Label4.Width       = 48;
  Label4.Height      = 8;
  Label4.Caption     = Label4_Caption;
  Label4.FontName    = Font_Glcd_Character8x7;
  Label4.FontWidth   = 8;
  Label4.FontHeight  = 7;
  Label4.FontOffset  = 32;
  Label4.FontColor   = _clDraw;
  Label4.Visible     = 1;
  Label4.OnUpPtr     = 0;
  Label4.OnDownPtr   = 0;
  Label4.OnClickPtr  = 0;
  Label4.OnPressPtr  = 0;

  Label5.OwnerScreen = &Screen1;
  Label5.Left        = 0;
  Label5.Top         = 24;
  Label5.Width       = 48;
  Label5.Height      = 8;
  Label5.Caption     = Label5_Caption;
  Label5.FontName    = Font_Glcd_Character8x7;
  Label5.FontWidth   = 8;
  Label5.FontHeight  = 7;
  Label5.FontOffset  = 32;
  Label5.FontColor   = _clDraw;
  Label5.Visible     = 1;
  Label5.OnUpPtr     = 0;
  Label5.OnDownPtr   = 0;
  Label5.OnClickPtr  = 0;
  Label5.OnPressPtr  = 0;

  Label3.OwnerScreen = &Screen1;
  Label3.Left        = 0;
  Label3.Top         = 32;
  Label3.Width       = 48;
  Label3.Height      = 8;
  Label3.Caption     = Label3_Caption;
  Label3.FontName    = Font_Glcd_Character8x7;
  Label3.FontWidth   = 8;
  Label3.FontHeight  = 7;
  Label3.FontOffset  = 32;
  Label3.FontColor   = _clDraw;
  Label3.Visible     = 1;
  Label3.OnUpPtr     = 0;
  Label3.OnDownPtr   = 0;
  Label3.OnClickPtr  = 0;
  Label3.OnPressPtr  = 0;

  Label6.OwnerScreen = &Screen1;
  Label6.Left        = 0;
  Label6.Top         = 58;
  Label6.Width       = 72;
  Label6.Height      = 8;
  Label6.Caption     = Label6_Caption;
  Label6.FontName    = Font_Glcd_Character8x7;
  Label6.FontWidth   = 8;
  Label6.FontHeight  = 7;
  Label6.FontOffset  = 32;
  Label6.FontColor   = _clDraw;
  Label6.Visible     = 1;
  Label6.OnUpPtr     = 0;
  Label6.OnDownPtr   = 0;
  Label6.OnClickPtr  = 0;
  Label6.OnPressPtr  = 0;

  Label7.OwnerScreen = &Screen1;
  Label7.Left        = 0;
  Label7.Top         = 40;
  Label7.Width       = 48;
  Label7.Height      = 8;
  Label7.Caption     = Label7_Caption;
  Label7.FontName    = Font_Glcd_Character8x7;
  Label7.FontWidth   = 8;
  Label7.FontHeight  = 7;
  Label7.FontOffset  = 32;
  Label7.FontColor   = _clDraw;
  Label7.Visible     = 1;
  Label7.OnUpPtr     = 0;
  Label7.OnDownPtr   = 0;
  Label7.OnClickPtr  = 0;
  Label7.OnPressPtr  = 0;

  Label8.OwnerScreen = &Screen1;
  Label8.Left        = 57;
  Label8.Top         = 8;
  Label8.Width       = 3;
  Label8.Height      = 8;
  Label8.Caption     = Label8_Caption;
  Label8.FontName    = Font_Glcd_Character8x7;
  Label8.FontWidth   = 8;
  Label8.FontHeight  = 7;
  Label8.FontOffset  = 32;
  Label8.FontColor   = _clDraw;
  Label8.Visible     = 1;
  Label8.OnUpPtr     = 0;
  Label8.OnDownPtr   = 0;
  Label8.OnClickPtr  = 0;
  Label8.OnPressPtr  = 0;

  Label9.OwnerScreen = &Screen1;
  Label9.Left        = 57;
  Label9.Top         = 16;
  Label9.Width       = 3;
  Label9.Height      = 8;
  Label9.Caption     = Label9_Caption;
  Label9.FontName    = Font_Glcd_Character8x7;
  Label9.FontWidth   = 8;
  Label9.FontHeight  = 7;
  Label9.FontOffset  = 32;
  Label9.FontColor   = _clDraw;
  Label9.Visible     = 1;
  Label9.OnUpPtr     = 0;
  Label9.OnDownPtr   = 0;
  Label9.OnClickPtr  = 0;
  Label9.OnPressPtr  = 0;

  Label10.OwnerScreen = &Screen1;
  Label10.Left        = 57;
  Label10.Top         = 24;
  Label10.Width       = 3;
  Label10.Height      = 8;
  Label10.Caption     = Label10_Caption;
  Label10.FontName    = Font_Glcd_Character8x7;
  Label10.FontWidth   = 8;
  Label10.FontHeight  = 7;
  Label10.FontOffset  = 32;
  Label10.FontColor   = _clDraw;
  Label10.Visible     = 1;
  Label10.OnUpPtr     = 0;
  Label10.OnDownPtr   = 0;
  Label10.OnClickPtr  = 0;
  Label10.OnPressPtr  = 0;

  Label11.OwnerScreen = &Screen1;
  Label11.Left        = 57;
  Label11.Top         = 32;
  Label11.Width       = 3;
  Label11.Height      = 8;
  Label11.Caption     = Label11_Caption;
  Label11.FontName    = Font_Glcd_Character8x7;
  Label11.FontWidth   = 8;
  Label11.FontHeight  = 7;
  Label11.FontOffset  = 32;
  Label11.FontColor   = _clDraw;
  Label11.Visible     = 1;
  Label11.OnUpPtr     = 0;
  Label11.OnDownPtr   = 0;
  Label11.OnClickPtr  = 0;
  Label11.OnPressPtr  = 0;

  Label12.OwnerScreen = &Screen1;
  Label12.Left        = 57;
  Label12.Top         = 40;
  Label12.Width       = 3;
  Label12.Height      = 8;
  Label12.Caption     = Label12_Caption;
  Label12.FontName    = Font_Glcd_Character8x7;
  Label12.FontWidth   = 8;
  Label12.FontHeight  = 7;
  Label12.FontOffset  = 32;
  Label12.FontColor   = _clDraw;
  Label12.Visible     = 1;
  Label12.OnUpPtr     = 0;
  Label12.OnDownPtr   = 0;
  Label12.OnClickPtr  = 0;
  Label12.OnPressPtr  = 0;

  Label13.OwnerScreen = &Screen1;
  Label13.Left        = 57;
  Label13.Top         = 0;
  Label13.Width       = 3;
  Label13.Height      = 8;
  Label13.Caption     = Label13_Caption;
  Label13.FontName    = Font_Glcd_Character8x7;
  Label13.FontWidth   = 8;
  Label13.FontHeight  = 7;
  Label13.FontOffset  = 32;
  Label13.FontColor   = _clDraw;
  Label13.Visible     = 1;
  Label13.OnUpPtr     = 0;
  Label13.OnDownPtr   = 0;
  Label13.OnClickPtr  = 0;
  Label13.OnPressPtr  = 0;

  CaptionButton1.OwnerScreen = &Screen1;
  CaptionButton1.Left        = 78;
  CaptionButton1.Top         = 3;
  CaptionButton1.Width       = 38;
  CaptionButton1.Height      = 16;
  CaptionButton1.BorderWidth = 3;
  CaptionButton1.Caption     = CaptionButton1_Caption;
  CaptionButton1.FontName    = Font_Glcd_5x7;
  CaptionButton1.FontWidth   = 5;
  CaptionButton1.FontHeight  = 7;
  CaptionButton1.FontOffset  = 32;
  CaptionButton1.FontColor   = _clInvert;
  CaptionButton1.Visible     = 1;
  CaptionButton1.OnUpPtr     = 0;
  CaptionButton1.OnDownPtr   = 0;
  CaptionButton1.OnClickPtr  = CaptionButton1Click;
  CaptionButton1.OnPressPtr  = 0;
}

static char IsInsideObject (unsigned int X, unsigned int Y, char Left, char Top, char Width, char Height) { // static
  if ( (Left<= X) && (Left+ Width - 1 >= X) &&
       (Top <= Y)  && (Top + Height - 1 >= Y) )
    return 1;
  else
    return 0;
}


#define GetLabel(index)               CurrentScreen->Labels[index]
#define GetCaptionButton(index)       CurrentScreen->CaptionButtons[index]


void DrawCaptionButton(TCaptionButton *Acaption_button) {
  Glcd_Box(Acaption_button->Left, Acaption_button->Top,
    Acaption_button->Left + Acaption_button->Width - 1,
    Acaption_button->Top + Acaption_button->Height - 1, _clDraw);
  if ((Acaption_button->Width >= 5) && (Acaption_button->Height >= 5))
    Glcd_Rectangle(Acaption_button->Left + 1, Acaption_button->Top + 1,
      Acaption_button->Left + Acaption_button->Width - 2,
      Acaption_button->Top + Acaption_button->Height - 2, _clClear);
  Glcd_Set_Font(Acaption_button->FontName, Acaption_button->FontWidth,
    Acaption_button->FontHeight, Acaption_button->FontOffset);
  Glcd_Write_Text(Acaption_button->Caption,
    Acaption_button->Left + Acaption_button->BorderWidth + 2,
    (Acaption_button->Top + Acaption_button->BorderWidth + 2) / 8,
    Acaption_button->FontColor);
}

static void DrawCaptionButtons() {
  int i;
  TCaptionButton *local_caption_button;

  for ( i = 0 ; i < CurrentScreen->CaptionButtonsCount ; i++ ) {
    local_caption_button = GetCaptionButton(i);
    if (local_caption_button->Visible == 1) {
      DrawCaptionButton(local_caption_button);
    }
  }
}


void DrawLabel(TLabel *Alabel) {
  Glcd_Set_Font(Alabel->FontName, Alabel->FontWidth,
    Alabel->FontHeight, Alabel->FontOffset);
  Glcd_Write_Text(Alabel->Caption, Alabel->Left,
    Alabel->Top / 8, Alabel->FontColor);
}

static void DrawLabels() {
  int i;
  TLabel *local_label;

  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (local_label->Visible == 1) {
      DrawLabel(local_label);
    }
  }
}

void DrawScreen(TScreen *aScreen) {
  Glcd_Fill(0x00);

  CurrentScreen = aScreen;

  DrawCaptionButtons();

  // draw labels last (always on top)
  DrawLabels();

}

static void Process_TP_Press(unsigned int X, unsigned int Y) {
  int i;
  TCaptionButton *local_caption_button;
  TLabel *local_label;

  // CaptionButtons
  for ( i = 0 ; i < CurrentScreen->CaptionButtonsCount ; i++ ) {
    local_caption_button = GetCaptionButton(i);
    if (IsInsideObject(X, Y, local_caption_button->Left, local_caption_button->Top,
                       local_caption_button->Width, local_caption_button->Height) == 1) {
      if (local_caption_button->OnPressPtr != 0) {
        local_caption_button->OnPressPtr();
        return;
      }
    }
  }

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                       local_label->Width, local_label->Height) == 1) {
      if (local_label->OnPressPtr != 0) {
        local_label->OnPressPtr();
        return;
      }
    }
  }

}

static void Process_TP_Up(unsigned int X, unsigned int Y) {
  int i;
  TCaptionButton *local_caption_button;
  TLabel *local_label;

  // CaptionButtons
  for ( i = 0 ; i < CurrentScreen->CaptionButtonsCount ; i++ ) {
    local_caption_button = GetCaptionButton(i);
    if (IsInsideObject(X, Y, local_caption_button->Left, local_caption_button->Top,
                       local_caption_button->Width, local_caption_button->Height) == 1) {
      if (local_caption_button->OnUpPtr != 0)
        local_caption_button->OnUpPtr();

      if (PressedObject == (void *)local_caption_button)
        if (local_caption_button->OnClickPtr != 0)
          local_caption_button->OnClickPtr();

      PressedObject = 0;
      return;

    }
  }

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                       local_label->Width, local_label->Height) == 1) {
      if (local_label->OnUpPtr != 0)
        local_label->OnUpPtr();

      if (PressedObject == (void *)local_label)
        if (local_label->OnClickPtr != 0)
          local_label->OnClickPtr();

      PressedObject = 0;
      return;

    }
  }

  PressedObject = 0;
}

static void Process_TP_Down(unsigned int X, unsigned int Y) {
  int i;
  TCaptionButton *local_caption_button;
  TLabel *local_label;

  // CaptionButtons
  for ( i = 0 ; i < CurrentScreen->CaptionButtonsCount ; i++ ) {
    local_caption_button = GetCaptionButton(i);
    if (IsInsideObject(X, Y, local_caption_button->Left, local_caption_button->Top,
                       local_caption_button->Width, local_caption_button->Height) == 1) {
      PressedObject = (void *)local_caption_button;
      if (local_caption_button->OnDownPtr != 0) {
        local_caption_button->OnDownPtr();
        return;
      }
    }
  }

  // Labels
  for ( i = 0 ; i < CurrentScreen->LabelsCount ; i++ ) {
    local_label = GetLabel(i);
    if (IsInsideObject(X, Y, local_label->Left, local_label->Top,
                       local_label->Width, local_label->Height) == 1) {
      PressedObject = (void *)local_label;
      if (local_label->OnDownPtr != 0) {
        local_label->OnDownPtr();
        return;
      }
    }
  }

}

void Check_TP() {
  if (TP_Press_Detect()) {
    // After a PRESS is detected read X-Y and convert it to 128x64 space
    if (TP_Get_Coordinates(&Xcoord, &Ycoord) == 0) {
      Process_TP_Press(Xcoord, Ycoord);
      if (PenDown == 0) {
        PenDown = 1;
        Process_TP_Down(Xcoord, Ycoord);
      }
    }
  }
  else if (PenDown == 1) {
    PenDown = 0;
    Process_TP_Up(Xcoord, Ycoord);
  }
}

void Start_TP() {
  InitializeTouchPanel();

  Glcd_Write_Text("CALIBRATION",32,3,1);
  Delay_ms(1000);
  Glcd_Fill(0);                                            // Clear GLCD
  Calibrate();

  Glcd_Fill(0);                                            // Clear GLCD
  InitializeObjects();
  DrawScreen(&Screen1);
}