#include "glcdtimers_objects.h"
#include "edt.h"
#define DOT_INVERT 2
//--------------------- User code ---------------------//
// Global date/time variables
char seconds, minutes, hours, day, month, year;
unsigned int adc_rd, adc_rd2;
int16_t idleCnt=0;
enum {T0, T1, T2, T3, T4 };
int evUART, evCircle, evADC, evRTC, evSnake, evProgress, evWalk;
XCircle c1, c2;
XFillRec fr1;
bit running;
char msg_rcv[8];

void T0_tick(unsigned char t, void* ev_arg);
void T1_tick(unsigned char t, void* ev_arg);
void T2_tick(unsigned char t, void* ev_arg);
void T3_tick(unsigned char t, void* ev_arg);
void T4_tick(unsigned char t, void* ev_arg);

void Idle(unsigned char t, void* ev_arg);
void EventUART(unsigned char t, void* ev_arg);
void EventCircle(unsigned char t, void* ev_arg);
void EventADC(unsigned char t, void* ev_arg);
void EventRTC(unsigned char t, void* ev_arg);
void EventSinSnake(unsigned char t, void* ev_arg);
void EventProgressBar(unsigned char t, void* ev_arg);
void EventSinWalk(unsigned char t, void* ev_arg);

// Functions
void TimersInit(void);
int Read_Time();
void Transform_Time();
void Display_Time();
void Start_EDT(void);
void SetRunning(char Value);
void LabelDisplayInt(int i, TLabel* lbl);
void LabelDisplayTxt(char* txt, TLabel* lbl);
void InitVar(void);

void TimersInit(void)
{
    T0CON=0xC6;
    TMR0L = 61;
    INTCON = 0xA0;
    INTCON.PEIE = 1;
}

//--------------------- Reads time and date information from RTC (PCF8583)
int Read_Time() {
  static char s=0;
  switch(s){
  case 0:
  I2C1_Start();               // Issue start signal
  s=1;
  break;
  case 1:
  I2C1_Wr(0xA0);           // Address PCF8583, see PCF8583 datasheet
  s=2;
  break;
  case 2:
  I2C1_Wr(2);              // Start from address 2
  s=3;
  break;
  case 3:
  I2C1_Repeated_Start();   // Issue repeated start signal
  s=4;
  break;
  case 4:
  I2C1_Wr(0xA1);           // Address PCF8583 for reading R/W=1
  s=5;
  break;
  case 5:
  seconds = I2C1_Rd(1);     // Read seconds byte
  s=6;
  break;
  case 6:
  minutes = I2C1_Rd(1);     // Read minutes byte
  s=7;
  break;
  case 7:
  hours = I2C1_Rd(1);       // Read hours byte
  s=8;
  break;
  case 8:
  day = I2C1_Rd(1);         // Read year/day byte
  s=9;
  break;
  case 9:
  month = I2C1_Rd(0);       // Read weekday/month byte
  s=10;
  break;
  case 10:
  I2C1_Stop();                // Issue stop signal
  s=0;
  break;
  default: break;
  }
  if(s==10) return 1;
  else return 0;
}

//-------------------- Formats date and time
void LabelDisplayInt(int i, TLabel* lbl)
{
    char txt[7];
    IntToStr(i, txt);
    strcpy(lbl->Caption, txt);
    DrawLabel(lbl);
}

void LabelDisplayTxt(char* txt, TLabel* lbl)
{
    strcpy(lbl->Caption, txt);
    DrawLabel(lbl);
}

void Transform_Time() {
  seconds  =  ((seconds & 0xF0) >> 4)*10 + (seconds & 0x0F);  // Transform seconds
  minutes  =  ((minutes & 0xF0) >> 4)*10 + (minutes & 0x0F);  // Transform months
  hours    =  ((hours & 0xF0)  >> 4)*10  + (hours & 0x0F);    // Transform hours
  year     =   (day & 0xC0) >> 6;                             // Transform year
  day      =  ((day & 0x30) >> 4)*10    + (day & 0x0F);       // Transform day
  month    =  ((month & 0x10)  >> 4)*10 + (month & 0x0F);     // Transform month
}

//-------------------- Output values to LCD
void Display_Time() {
   char txt[7];
   txt[0] = (hours / 10)   + 48;
   txt[1] = (hours % 10)   + 48;
   txt[2] = ':';
   txt[3] = (minutes / 10) + 48;
   txt[4] = (minutes % 10) + 48;
   txt[5] = ':';
   txt[6] = (seconds / 10) + 48;
   txt[7] = (seconds % 10) + 48;
   txt[8] = '\0';
   LabelDisplayTxt(txt, &Label6);
}

void ADRequest(unsigned char ch){
   ch<<=2;
   ADCON0=0x01;
   ADCON0 |= ch;
   ADCON0.GO = 1;
}

void RS232Init1(void){
   SPBRG1 = 15;
   // send
   //TXSTA1.SPEN=1;
   //TXSTA1.CREN=1;
   //TXSTA1.SYNC=0;
   //PIE3.TX1IE=1;
   //PIR3.TX1IF=0;
   // receive
}

void RS232Init2(void){
   SPBRG2 = 15;
   // send
   RCSTA2.SPEN=1;
   RCSTA2.CREN=1;
   PIE3.RC2IE=1;
   PIR3.RC2IF=0;
}

void RS232_Send(void* ev_arg){
   TXSTA1.TXEN=1;
   TXREG1=49;
}

void interrupt (void)
{
   static int16_t i = 0, i2 = 0;
   if(INTCON.TMR0IF)
   {
      TimerForDispatcher();
      TMR0L = 61;
      INTCON.T0IF =0 ;
   }
   else if(PIR3.RC2IF)
   {  // receiver
      msg_rcv[i]=RCREG2;
      if(i<5){
         i++;
      }
      else{
         msg_rcv[6]='\0';
         i=0;
         EventIPulse(evUART, 0,&msg_rcv);
      }
      PIR3.RC2IF=0;
   }
   /*
   else if(PIR3.TX1IF){ // sender
      if(i2<5){
         TXREG1 = 48+i2; i2++;
      }
      else{
         TXSTA1.TXEN=0; i2=0;
      }
      PIR3.TX1IF=0;
   }
   */
}

void T0_tick(unsigned char t, void* ev_arg)
{
    static int16_t i = 0;
    LabelDisplayInt(i++, &Label1);
    PORTB.F0 = ~PORTB.F0;
}

void T1_tick(unsigned char t, void* ev_arg)
{
    static int16_t i = 0;
    LabelDisplayInt(i++, &Label2);
    PORTB.F1 = ~PORTB.F1;
    EventPulse(evWalk,0,0);
}

void T2_tick(unsigned char t, void* ev_arg)
{
    LabelDisplayInt(idleCnt, &Label3);
    PORTB.F2 = ~PORTB.F2;
    idleCnt=0;
}

void T3_tick(unsigned char t, void* ev_arg)
{
   static ch = 32;
   adc_rd2 = ADC_Get_Sample(4);
   adc_rd = ADC_Get_Sample(2);
   EventPulse(evADC,0,0);
   EventPulse(evRTC,0,0);

   UART1_Write(ch++);
   if(ch==128) ch=32;

   //RS232_Send(0);
}

void T4_tick(unsigned char t, void* ev_arg)
{
   EventPulse(evSnake,0,0);
   EventPulse(evProgress, 0, &fr1);
}

void Idle(unsigned char t, void* ev_arg)
{
   static int16_t i=0;
   idleCnt++;

   /*
   if (UART2_Data_Ready() == 1) 
   {
      msg_rcv[i]=UART2_Read();
      msg_rcv[++i]='\0';
      if(i==6)
      {
         i=0;
         EventPulse(evUART, 0, 0);
      }   }
   */
}

void EventUART(unsigned char t, void* ev_arg)
{
    LabelDisplayTxt(msg_rcv, &Label7);
}


void EventCircle(unsigned char t, void* ev_arg)
{
    int m,xc,yc,r,i,s;
    float mr;
    XCircle* c;
    c=(XCircle*)ev_arg;
    xc=c->xc; yc=c->yc; r=c->r; i=c->i; s=c->s;
    if(i<0.72*r){
          mr=sqrt(r*r-i*i);
          m=(int) mr;
          if(fabs(m-mr)>0.5) m++;

          Glcd_Dot(xc+m, yc+i, s);
          Glcd_Dot(xc+m, yc-i, s);
          Glcd_Dot(xc-m, yc+i, s);
          Glcd_Dot(xc-m, yc-i, s);

          Glcd_Dot(xc+i, yc+m, s);
          Glcd_Dot(xc+i, yc-m, s);
          Glcd_Dot(xc-i, yc+m, s);
          Glcd_Dot(xc-i, yc-m, s);

          (c->i)++;
          EventPulse(evCircle,0,ev_arg);
        }
    else{
          Glcd_Dot(xc+r, yc, s);
          Glcd_Dot(xc-r, yc, s);
          Glcd_Dot(xc, yc+r, s);
          Glcd_Dot(xc, yc-r, s);

          if(s==1){
             c->s=0;
          }
          else{
             c->s=1;
             //if((c->xc) <110) (c->xc)+=2;
          }
          c->i=1;
    }
}

void EventADC(unsigned char t, void* ev_arg)
{
    LabelDisplayInt(adc_rd, &Label5);
    LabelDisplayInt(adc_rd2, &Label4);
}

void EventRTC(unsigned char t, void* ev_arg)
{
    static char s=0;
    switch(s){
    case 0: if(Read_Time()) s=1;
            EventPulse(evRTC,0,0);
            break;
    case 1: Transform_Time();
            s=2;
            EventPulse(evRTC,0,0);
            break;
    case 2: Display_Time();
            s=0;
            break;
    default: break;
    }
}

void EventSinSnake(unsigned char t, void* ev_arg)
{
    int     xc=79, yc=30, m;
    static int s=1;
    static double x=0;
    if(x <= 41)
   {
      double mr;
      mr=7 * sin(-x/6);
      m=(int) mr;
      Glcd_Dot(xc+(int) x, yc+m , s);
      x=x+0.4;
      EventPulse(evSnake,0,0);
    }
    else{
      x=0;
      if(s==1) s=0;
      else s=1;
    }
}

void EventProgressBar(unsigned char t, void* ev_arg){
   int x,y,a,b,i,j,s;
   XFillRec* fr;
   fr=(XFillRec*) ev_arg;
   x=fr->x;   y=fr->y;
   a=fr->a;   b=fr->b;
   i=fr->i;   j=fr->j;

   if(i < a){
      if(j < b){
         Glcd_Dot(x+i,y+j,DOT_INVERT);
         (fr->j)++;
      }
      else{
         (fr->i)++;
         (fr->j)=0;
      }
      EventPulse(evProgress, 0, ev_arg);
   }
   else{
      (fr->i)=0; (fr->j)=0; (fr->s)=0;
   }
}

void EventSinWalk(unsigned char t, void* ev_arg)
{
    int     xc=79, yc=52;
    static int s=1;
    static double x=0, m=0;
    if(x <= 37)
   {
      if(s==1 ) {
         double mr;
         mr=7 * sin(-x/6);
         m=(int) mr;
         Glcd_Box(xc+(int) x, yc+m , xc+(int) x+3, yc+m+3, s);
         if(x==16 || x==22) EventPulse(evCircle, 0, &c1);
         s=0;
      }
      else {
         Glcd_Box(xc+(int) x, yc+m , xc+(int) x+3, yc+m+3, s);
         s=1;
         x=x+1;
      }
    }
    else{
      x=0;
    }
}

void SetRunning(char Value) {
  if (Value == 0) {
    running = 0;
    strcpy(CaptionButton1_Caption, "START");
    TimerStop(T0);
    TimerStop(T1);
    //TimerStop(T2);
    TimerStop(T3);
    TimerStop(T4);
  }
  else {
    running = 1;
    strcpy(CaptionButton1_Caption, "STOP");
    TimerStart(T0, 10);
    TimerStart(T1, 20);
    //TimerStart(T2, 300);
    TimerStart(T3, 100);
    TimerStart(T4, 400);
    EventPulse(evSnake,0,0);
  }
  DrawCaptionButton(&CaptionButton1);
}
//----------------- End of User code ------------------//

// Event Handlers
void CaptionButton1Click() {
  if (running == 1)
    SetRunning(0);
  else
    SetRunning(1);
}

void InitVar(void)
{
   c1.xc=100; c1.yc=52; c1.r=8; c1.s=1; c1.i=1;
   //c2.xc=100; c2.yc=52; c2.r=7; c2.s=1; c2.i=1;
   fr1.x=0; fr1.y=52; fr1.a=68; fr1.b=2; fr1.i=0; fr1.j=0; fr1.s=0;
   Glcd_Rectangle(78,20,120,40,1);
   Glcd_Rectangle(78,42,120,62,1);
   running = 0;
}

void Start_EDT(void)
 {
   TRISB = 0x00;
   PORTB = 0x00;


   //RS232Init1();
   
   UART1_Init(9600);
   Delay_ms(200);
   //UART2_Init(9600);
   
   RS232Init2();
   Delay_ms(200);
   
   EventsInit();
   TimersInit();
   TimerCreate(T0, T0_tick);
   TimerCreate(T1, T1_tick);
   TimerCreate(T2, T2_tick);
   TimerCreate(T3, T3_tick);
   TimerCreate(T4, T4_tick);

   evUART =     EventCreate(EventUART, 0, 0);
   evCircle =   EventCreate(EventCircle, 0, 0);
   evADC =      EventCreate(EventADC, 0, 0);
   evRTC =      EventCreate(EventRTC, 0, 0);
   evSnake =    EventCreate(EventSinSnake, 0, 0);
   evProgress = EventCreate(EventProgressBar, 0, 0);
   evWalk =     EventCreate(EventSinWalk, 0, 0);

   InitVar();
   I2C1_Init(100000);
   IdleSet(Idle);
   TimerStart(T2, 100);
}